# 即时动画机

Milease 为`object`类型提供了一系列的拓展函数，包括`Milease`、`MileaseTo`、`MileaseAdditive`等。

## 生成即时动画机

### 方式一：使用 DSL 生成（推荐）

相比传统的生成方式，我们注意到，开发者更多时候需要指定缓动函数，而缓动函数位于可选参数表有些靠后的位置，也不便于开发者观察开始值、终止值、动画长度和开始时间，特别是在没有 `Inlay Hints` 支持的编辑器上。

我们使用 `T4模板` 来生成针对 `object` 的一系列动画拓展函数，它遵循以下规律：

```c#
MilInstantAnimator object.M[缓动函数名][缓动分支名]<T,E>(Expression<Func<T, E>> mbExpr, AniExpression<E> aniExpr);
```

或

```c#
MilInstantAnimator object.M[缓动函数名][缓动分支名]<T,E>(Expression<Func<T, E>> mbExpr, E from, E to);
```

举个例子，当你需要使用 `Quad` `Out` 缓动时，相应的函数为：

```c#
MilInstantAnimator object.MQuadOut<T,E>(Expression<Func<T, E>> mbExpr, E from, E to);
```

同时，我们对其做了一系列运算符重载，你可以通过这个方式生成一个动画机/动画机片段。

当你需要将其作为一整个动画机直接使用时，我们推荐以下写法：

```c#
// 两种方式均可
AudioSource.MQuadOut(x => x.volume, [开始时间] + [动画时长] / [开始值].To([终止值])).Play();
([开始时间] + [动画时长] / AudioSource.MQuadOut(x => x.volume, [开始值], [终止值])).Play();
```

当你需要将音量从当前音量过渡到目标音量，而不是从一个指定的音量开始时，你可以使用以下写法：

```c#
// 两种方式均可
AudioSource.MQuadOut(x => x.volume, [开始时间] + [动画时长] / [目标音量].ToThis()).Play();
([开始时间] + [动画时长] / AudioSource.MQuadOut(x => x.volume, [目标音量].ToThis())).Play();
```

当你需要将其作为动画机片段组合时，我们推荐以下写法：

```c#
[开始时间] + [动画时长] / AudioSource.MQuadOut(x => x.volume, [开始值], [终止值])
```

或以下写法：

```c#
[开始时间] + [动画时长] / AudioSource.MQuadOut(x => x.volume, [目标音量])
```

### 方式二：使用传统方式生成

```c#
MilInstantAnimator object.Milease(string memberName, object startValue, object toValue, float duration);
```

该函数将生成一个动画机，将 `object.memberName` 在 `duration` 的时间内 由 `startValue` 过渡到 `toValue`。

`duration` 以秒为单位，Milease 中所有的时间参数均以秒为单位。

```c#
MilInstantAnimator object.MileaseTo(string memberName, object toValue, float duration);
```

该函数将生成一个动画机，将 `object.memberName` 在 `duration` 的时间内 由 当前的值 过渡到 `toValue`。

```c#
MilInstantAnimator object.MileaseAdditive(string memberName, object startValue, object toValue, float duration);
```

该函数将生成一个动画机，将 `object.memberName` 在 `duration` 的时间内 由 `当前值 + startValue` 过渡到 `当前值 + toValue`。

动画机生成后**不会**立即播放。

## 即时动画机的限制条件

* `memberName` 是 大小写**敏感**的。

* 指定的成员必须可写，且必须是公有成员，不能是静态成员。

* 指定的成员可以是字段，也可以是属性。
* 指定的成员类型必须符合以下其中一个特性：
  * 类型是基元类型，例如：bool、int、float、double、string 等。
  * 类型可序列化，并已重载这些运算符：`type + type`、`type - type`、`type * float`

## 即时动画机的生命周期

默认情况下，它会随场景销毁而被停止。你可以使用如下代码改变这一行为，让它在场景被卸载时仍然继续播放：

```
animator.DontStopOnLoad();
```

但请注意，若动画机正在操作的物体因场景销毁而被销毁，则会发生异常。

因此，可能需要手动在物体被销毁时停止动画机的运行：

```c#
animator.Stop();
```

动画机不需要手动释放资源。

## 即时动画机的重置模式

即时动画机的重置模式包含：

1. 重置至原始状态**（默认）**

   重置时，将撤销动画机对物体进行的修改，还原至动画机对物体相应状态进行修改时的状态。

2. 重置至动画设置的初始状态

   重置时，将物体状态重置为动画机中，各种状态在动画机时间轴上最早时间的状态。

由`MileaseTo`、`MileaseAdditive`创建的动画无法通过 方式二 重置，它们会强制使用 方式一 重置，因为它们的动画设定中不存在对其最早时间状态的设定。

改变动画机的重置模式：

```c#
animator.UsingResetMode(mode);
```

当 使用 方式二 重置时，动画机在播放前将先执行重置，再播放。

## 即时动画机的使用

开始动画机的播放：

```c#
animator.Play();
```

暂停播放：

```c#
animator.Pause();
```

停止播放：

```c#
animator.Stop();
```

重置状态：

```c#
animator.Reset(mode);
```

## 自定义动画处理函数

通过实现以下委托来实现：

```c#
delegate void MileaseHandleFunction(MilHandleFunctionArgs e);
```

调用`e.GetTarget<T>()` 获取目标物体，`e.Progress` 为动画进度（经过缓动处理后的进行），范围大约是[0.0, 1.0]。

使用`Milease`创建自定义动画：

```c#
MilInstantAnimator object.Milease(MileaseHandleFunction animationFunction, MileaseHandleFunction resetFunction, float duration);
```

其中，`resetFunction` 可以为null，该函数将在动画机执行重置时调用。

## 关键帧事件

通过 `Action` 的拓展函数构造：

```c#
MilInstantAnimator Action.AsMileaseKeyEvent(float delay = 0);
```

## 嵌套的复杂动画

利用 `animator.While`、`animator.Then ` 等函数嵌套定义动画，例如：

```c#
var spriteRenderer = GetComponent<SpriteRenderer>();

Animation =
    MilInstantAnimator.Start(
        1f / transform.Milease(x => x.position,new Vector3(0f, 0f, 0f), new Vector3(1f, 1f, 0f))
    )
    .And(
        0.5f + 1f / spriteRenderer.Milease(x => x.color, Color.white, Color.red)
    )
    .Then(
        1f / transform.MBounce(x => transform.localScale, new Vector3(1f, 1f, 1f), new Vector3(2f, 2f, 2f))
    ).Delayed(1f)
    .Then(
        1f / Text.MLinear(x => x.text, "Start!", "Finish!")
    )
    .Then(
        Text.Milease((e) =>
        {
            var text = e.GetTarget<TMP_Text>();
            text.text = $"Hide after {((1f - e.Progress) * 2f):F1}s...";
        }, null,2f, 0f, EaseFunction.Linear)
    )
    .Then(
        Text.gameObject.Milease(HandleFunction.Hide, HandleFunction.AutoActiveReset(Text.gameObject), 0f)
    )
    .Then(
        1f / transform.Milease(x => x.position, Vector3.zero.ToThis())
    );
```

与上一个动画同时播放指定动画：

```c#
MilInstantAnimator animator.While(MilInstantAnimation animation);
```

与上一个动画同时播放一组指定的动画：

```c#
MilInstantAnimator animator.While(params MilInstantAnimation[] animations);
```

在上一个/组动画全部播放完毕后，播放指定动画：

```c#
MilInstantAnimator animator.Then(MilInstantAnimation animation);
```

在上一个/组动画全部播放完毕后，播放指定一组动画：

```c#
MilInstantAnimator animator.Then(params MilInstantAnimation[] animations);
```

在上一个/组动画全部播放完毕后，按顺序播放一系列动画：

```c#
MilInstantAnimator animator.ThenOneByOne(params MilInstantAnimation[] animations);
```

设定上一个/组动画的每个动画的延后播放时间：

```c#
MilInstantAnimator animator.Delayed(float time);
```

